module NineOneOne
  class PagerDutyService
    BASE_HOST = 'events.pagerduty.com'.freeze
    EVENTS_API_V2_ENDPOINT = '/v2/enqueue'.freeze
    THROTTLE_HTTP_STATUS = 403
    THROTTLE_RETRIES = 2
    HIGH_URGENCY_ERROR = 'error'.freeze
    LOW_URGENCY_ERROR = 'warning'.freeze

    def initialize(api_integration_key)
      @api_integration_key = api_integration_key
      @http = Http.new(BASE_HOST)
    end

    def trigger_event(description, source: Socket.gethostname, dedup_key: nil, severity: HIGH_URGENCY_ERROR,
                      details_hash: nil)
      make_request(request_body('trigger', dedup_key, description, source, severity, details_hash))
    end

    def resolve_event(dedup_key)
      make_request(request_body('resolve', dedup_key))
    end

    private

    attr_reader :api_integration_key, :http

    def make_request(request_body)
      headers = {'Content-Type' => 'application/json'}
      response = nil

      retry_on(THROTTLE_HTTP_STATUS, THROTTLE_RETRIES) do
        response = http.post(EVENTS_API_V2_ENDPOINT, request_body, headers)
        response.code.to_i
      end

      # rubocop:disable Style/GuardClause
      unless response.is_a?(Net::HTTPSuccess)
        raise IncidentReportingError, "Failed to send an event to PagerDuty: #{response.body} #{request_body}"
      end
      # rubocop:enable Style/GuardClause
    end

    def retry_on(value, retries_number)
      retry_number = 0

      while yield == value && retry_number <= retries_number
        retry_number += 1
        sleep 2**retry_number
      end
    end

    def request_body(event_action, dedup_key, description = nil, source = nil, severity = nil, details_hash = nil)
      {
        routing_key: api_integration_key,
        event_action: event_action,
        dedup_key: dedup_key,
        payload: {
          summary: description,
          source: source,
          severity: severity,
          custom_details: details_hash
        }.reject { |_, v| v.nil? }
      }.reject { |_, v| v.nil? || v.empty? }.to_json
    end
  end
end
